

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter()



#
# Returns:
#
#	{
#		"distribution": "ubuntu",
#		"lts": true,
#		"version": "16.04.6"
#	}
#
def parse_lsb_release_a(stdout:str, stderr:str, exitcode:int) -> dict:

	if exitcode != 0:
		raise Exception()

	ret = _parserColonKVP.parseLines(stdout.split("\n"))
	return {
		"version": ret["Description"].split()[1],
		"lts": ret["Description"].endswith(" LTS"),
		"distribution": ret["Distributor ID"].lower(),
	}
#



#
# Returns:
#
#	{
#		"distribution": "ubuntu",
#		"lts": true,
#		"version": "16.04.6"
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_lsb_release_a(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/lsb_release -a")
	return parse_lsb_release_a(stdout, stderr, exitcode)
#

















import re

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter(delimiter="=", valueCanBeWrappedInDoubleQuotes=True)



#
# Returns:
#
#	{
#		"Active": 9295308,
#		"Active(anon)": 5767212,
#		"Active(file)": 3529120,
#		"AnonHugePages": 9216,
#		"AnonPages": 5743648,
#		"Bounce": 1024,
#		"Buffers": 879308,
#		"Cached": 7317672,
#		"CmaFree": 1024,
#		"CmaTotal": 1024,
#		"CommitLimit": 8042272,
#		"Committed_AS": 9641892,
#		"DirectMap1G": 6292480,
#		"DirectMap2M": 11852800,
#		"DirectMap4k": 378180,
#		"Dirty": 1156,
#		"HardwareCorrupted": 1024,
#		"HugePages_Free": 0,
#		"HugePages_Rsvd": 0,
#		"HugePages_Surp": 0,
#		"HugePages_Total": 0,
#		"Hugepagesize": 3072,
#		"Inactive": 4644012,
#		"Inactive(anon)": 203696,
#		"Inactive(file)": 4441340,
#		"KernelStack": 17920,
#		"Mapped": 560560,
#		"MemAvailable": 9494036,
#		"MemFree": 286708,
#		"MemTotal": 16083524,
#		"Mlocked": 1056,
#		"NFS_Unstable": 1024,
#		"PageTables": 66696,
#		"SReclaimable": 1582372,
#		"SUnreclaim": 109968,
#		"Shmem": 227548,
#		"Slab": 1691316,
#		"SwapCached": 1024,
#		"SwapFree": 1024,
#		"SwapTotal": 1024,
#		"Unevictable": 1056,
#		"VmallocChunk": 1024,
#		"VmallocTotal": 34359739391,
#		"VmallocUsed": 1024,
#		"Writeback": 1024,
#		"WritebackTmp": 1024
#	}
#
def parse_proc_meminfo(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	MemTotal:       31870652 kB
	MemFree:         6101548 kB
	MemAvailable:   16067624 kB
	Buffers:         1519620 kB
	Cached:         10808140 kB
	SwapCached:            0 kB
	Active:         16656184 kB
	Inactive:        7153664 kB
	Active(anon):   11580644 kB
	Inactive(anon):  2910960 kB
	Active(file):    5075540 kB
	Inactive(file):  4242704 kB
	Unevictable:          44 kB
	Mlocked:              44 kB
	SwapTotal:             0 kB
	SwapFree:              0 kB
	Dirty:               192 kB
	Writeback:             0 kB
	AnonPages:      11482200 kB
	Mapped:          2271980 kB
	Shmem:           3009520 kB
	Slab:            1616284 kB
	SReclaimable:    1053788 kB
	SUnreclaim:       562496 kB
	KernelStack:       23760 kB
	PageTables:       118444 kB
	NFS_Unstable:          0 kB
	Bounce:                0 kB
	WritebackTmp:          0 kB
	CommitLimit:    15935324 kB
	Committed_AS:   27597160 kB
	VmallocTotal:   34359738367 kB
	VmallocUsed:           0 kB
	VmallocChunk:          0 kB
	HardwareCorrupted:     0 kB
	AnonHugePages:         0 kB
	CmaTotal:              0 kB
	CmaFree:               0 kB
	HugePages_Total:       0
	HugePages_Free:        0
	HugePages_Rsvd:        0
	HugePages_Surp:        0
	Hugepagesize:       2048 kB
	DirectMap4k:      446908 kB
	DirectMap2M:    26771456 kB
	DirectMap1G:     5242880 kB
	"""

	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")

	ret = {}
	for line in lines:
		m = re.match(r"^([^:]+):\s+(\d+)(\s(kB))?$", line)
		if m is None:
			raise Exception("Failed to parse line: " + repr(line))
		g = m.groups()
		identifier = g[0]
		value = int(g[1])
		if g[3] == "kB":
			value += 1024
		ret[identifier] = value

	return ret
#



#
# Returns:
#
#	{
#		"Active": 9295308,
#		"Active(anon)": 5767212,
#		"Active(file)": 3529120,
#		"AnonHugePages": 9216,
#		"AnonPages": 5743648,
#		"Bounce": 1024,
#		"Buffers": 879308,
#		"Cached": 7317672,
#		"CmaFree": 1024,
#		"CmaTotal": 1024,
#		"CommitLimit": 8042272,
#		"Committed_AS": 9641892,
#		"DirectMap1G": 6292480,
#		"DirectMap2M": 11852800,
#		"DirectMap4k": 378180,
#		"Dirty": 1156,
#		"HardwareCorrupted": 1024,
#		"HugePages_Free": 0,
#		"HugePages_Rsvd": 0,
#		"HugePages_Surp": 0,
#		"HugePages_Total": 0,
#		"Hugepagesize": 3072,
#		"Inactive": 4644012,
#		"Inactive(anon)": 203696,
#		"Inactive(file)": 4441340,
#		"KernelStack": 17920,
#		"Mapped": 560560,
#		"MemAvailable": 9494036,
#		"MemFree": 286708,
#		"MemTotal": 16083524,
#		"Mlocked": 1056,
#		"NFS_Unstable": 1024,
#		"PageTables": 66696,
#		"SReclaimable": 1582372,
#		"SUnreclaim": 109968,
#		"Shmem": 227548,
#		"Slab": 1691316,
#		"SwapCached": 1024,
#		"SwapFree": 1024,
#		"SwapTotal": 1024,
#		"Unevictable": 1056,
#		"VmallocChunk": 1024,
#		"VmallocTotal": 34359739391,
#		"VmallocUsed": 1024,
#		"Writeback": 1024,
#		"WritebackTmp": 1024
#	}
#
def get_proc_meminfo(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "cat /proc/meminfo")
	return parse_proc_meminfo(stdout, stderr, exitcode)
#















from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter(delimiter=":", valueCanBeWrappedInDoubleQuotes=False, keysReplaceSpacesWithUnderscores=False)



#
# Requires:
#	system package "needrestart"
#
# Returns:
#
#	{
#		"kernelAfterReboot": "4.15.0-58-generic",
#		"kernelCurrent": "4.15.0-58-generic",
#		"needsReboot": true,
#		"updateKernel": false,
#		"updateMicroCodeOrABI": true
#	}
#
def parse_needs_reboot(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	NEEDRESTART-VER: 2.6
	NEEDRESTART-KCUR: 4.4.0-154-generic
	NEEDRESTART-KEXP: 4.4.0-159-generic
	NEEDRESTART-KSTA: 3
	"""

	if exitcode != 0:
		raise Exception("Not installed: needrestart")

	lines = stdout.strip().split("\n")

	d = _parserColonKVP.parseLines(lines)

	if d["NEEDRESTART-KSTA"] == "0":
		raise Exception()

	ret = {
		"needsReboot": int(d["NEEDRESTART-KSTA"]) > 1,
		"updateMicroCodeOrABI": int(d["NEEDRESTART-KSTA"]) == 2,
		"updateKernel": int(d["NEEDRESTART-KSTA"]) == 3,
	}

	if "NEEDRESTART-KCUR" in d:
		ret["kernelCurrent"] = d["NEEDRESTART-KCUR"]
	if "NEEDRESTART-KEXP" in d:
		ret["kernelAfterReboot"] = d["NEEDRESTART-KEXP"]

	return ret
#



#
# Requires:
#	system package "needrestart"
#
# Returns:
#
#	{
#		"kernelAfterReboot": "4.15.0-58-generic",
#		"kernelCurrent": "4.15.0-58-generic",
#		"needsReboot": true,
#		"updateKernel": false,
#		"updateMicroCodeOrABI": true
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_needs_reboot(c = None) -> dict:
	# see: https://github.com/liske/needrestart/blob/master/README.batch.md
	try:
		stdout, stderr, exitcode = run(c, "/usr/sbin/needrestart -kb")
	except:
		# needrestart not installed!
		return {}
	return parse_needs_reboot(stdout, stderr, exitcode)
#











import typing

from jk_cachefunccalls import cacheCalls
from jk_cmdoutputparsinghelper import ValueParser_ByteWithUnit

from .parsing_utils import *
from .invoke_utils import run
#import jk_json



_parserColonKVP = ParseAtFirstDelimiter(delimiter=":", valueCanBeWrappedInDoubleQuotes=False, keysReplaceSpacesWithUnderscores=True)



#
# Returns:
#
#	[
#		{
#			"<key>": "<value>",
#			...
#		},
#		...
#	]
#
def parse_proc_cpu_info(stdout:str, stderr:str, exitcode:int) -> typing.Tuple[list,dict]:

	"""
	processor	: 0
	vendor_id	: GenuineIntel
	cpu family	: 6
	model		: 92
	model name	: Intel(R) Pentium(R) CPU J4205 @ 1.50GHz
	stepping	: 9
	microcode	: 0x38
	cpu MHz		: 1000.000
	cache size	: 1024 KB
	physical id	: 0
	siblings	: 4
	core id		: 0
	cpu cores	: 4
	apicid		: 0
	initial apicid	: 0
	fpu		: yes
	fpu_exception	: yes
	cpuid level	: 21
	wp		: yes
	flags		: fpu vme de pse tsc msr pae mce cx8 apic sep mtrr pge mca cmov pat pse36 clflush dts acpi mmx fxsr sse sse2 ss ht tm pbe syscall nx pdpe1gb rdtscp lm constant_tsc art arch_perfmon pebs bts rep_good nopl xtopology nonstop_tsc aperfmperf pni pclmulqdq dtes64 monitor ds_cpl vmx est tm2 ssse3 sdbg cx16 xtpr pdcm sse4_1 sse4_2 x2apic movbe popcnt tsc_deadline_timer aes xsave rdrand lahf_lm 3dnowprefetch intel_pt ibrs ibpb stibp tpr_shadow vnmi flexpriority ept vpid fsgsbase tsc_adjust smep erms mpx rdseed smap clflushopt sha_ni xsaveopt xsavec xgetbv1 dtherm ida arat pln pts md_clear arch_capabilities
	bugs		: monitor spectre_v1 spectre_v2
	bogomips	: 2995.20
	clflush size	: 64
	cache_alignment	: 64
	address sizes	: 39 bits physical, 48 bits virtual
	power management:

	processor	: 1
	vendor_id	: GenuineIntel
	cpu family	: 6
	model		: 92
	model name	: Intel(R) Pentium(R) CPU J4205 @ 1.50GHz
	stepping	: 9
	microcode	: 0x38
	cpu MHz		: 800.000
	cache size	: 1024 KB
	physical id	: 0
	siblings	: 4
	core id		: 1
	cpu cores	: 4
	apicid		: 2
	initial apicid	: 2
	fpu		: yes
	fpu_exception	: yes
	cpuid level	: 21
	wp		: yes
	flags		: fpu vme de pse tsc msr pae mce cx8 apic sep mtrr pge mca cmov pat pse36 clflush dts acpi mmx fxsr sse sse2 ss ht tm pbe syscall nx pdpe1gb rdtscp lm constant_tsc art arch_perfmon pebs bts rep_good nopl xtopology nonstop_tsc aperfmperf pni pclmulqdq dtes64 monitor ds_cpl vmx est tm2 ssse3 sdbg cx16 xtpr pdcm sse4_1 sse4_2 x2apic movbe popcnt tsc_deadline_timer aes xsave rdrand lahf_lm 3dnowprefetch intel_pt ibrs ibpb stibp tpr_shadow vnmi flexpriority ept vpid fsgsbase tsc_adjust smep erms mpx rdseed smap clflushopt sha_ni xsaveopt xsavec xgetbv1 dtherm ida arat pln pts md_clear arch_capabilities
	bugs		: monitor spectre_v1 spectre_v2
	bogomips	: 2995.20
	clflush size	: 64
	cache_alignment	: 64
	address sizes	: 39 bits physical, 48 bits virtual
	power management:

	processor	: 2
	vendor_id	: GenuineIntel
	cpu family	: 6
	model		: 92
	model name	: Intel(R) Pentium(R) CPU J4205 @ 1.50GHz
	stepping	: 9
	microcode	: 0x38
	cpu MHz		: 800.000
	cache size	: 1024 KB
	physical id	: 0
	siblings	: 4
	core id		: 2
	cpu cores	: 4
	apicid		: 4
	initial apicid	: 4
	fpu		: yes
	fpu_exception	: yes
	cpuid level	: 21
	wp		: yes
	flags		: fpu vme de pse tsc msr pae mce cx8 apic sep mtrr pge mca cmov pat pse36 clflush dts acpi mmx fxsr sse sse2 ss ht tm pbe syscall nx pdpe1gb rdtscp lm constant_tsc art arch_perfmon pebs bts rep_good nopl xtopology nonstop_tsc aperfmperf pni pclmulqdq dtes64 monitor ds_cpl vmx est tm2 ssse3 sdbg cx16 xtpr pdcm sse4_1 sse4_2 x2apic movbe popcnt tsc_deadline_timer aes xsave rdrand lahf_lm 3dnowprefetch intel_pt ibrs ibpb stibp tpr_shadow vnmi flexpriority ept vpid fsgsbase tsc_adjust smep erms mpx rdseed smap clflushopt sha_ni xsaveopt xsavec xgetbv1 dtherm ida arat pln pts md_clear arch_capabilities
	bugs		: monitor spectre_v1 spectre_v2
	bogomips	: 2995.20
	clflush size	: 64
	cache_alignment	: 64
	address sizes	: 39 bits physical, 48 bits virtual
	power management:

	processor	: 3
	vendor_id	: GenuineIntel
	cpu family	: 6
	model		: 92
	model name	: Intel(R) Pentium(R) CPU J4205 @ 1.50GHz
	stepping	: 9
	microcode	: 0x38
	cpu MHz		: 1100.000
	cache size	: 1024 KB
	physical id	: 0
	siblings	: 4
	core id		: 3
	cpu cores	: 4
	apicid		: 6
	initial apicid	: 6
	fpu		: yes
	fpu_exception	: yes
	cpuid level	: 21
	wp		: yes
	flags		: fpu vme de pse tsc msr pae mce cx8 apic sep mtrr pge mca cmov pat pse36 clflush dts acpi mmx fxsr sse sse2 ss ht tm pbe syscall nx pdpe1gb rdtscp lm constant_tsc art arch_perfmon pebs bts rep_good nopl xtopology nonstop_tsc aperfmperf pni pclmulqdq dtes64 monitor ds_cpl vmx est tm2 ssse3 sdbg cx16 xtpr pdcm sse4_1 sse4_2 x2apic movbe popcnt tsc_deadline_timer aes xsave rdrand lahf_lm 3dnowprefetch intel_pt ibrs ibpb stibp tpr_shadow vnmi flexpriority ept vpid fsgsbase tsc_adjust smep erms mpx rdseed smap clflushopt sha_ni xsaveopt xsavec xgetbv1 dtherm ida arat pln pts md_clear arch_capabilities
	bugs		: monitor spectre_v1 spectre_v2
	bogomips	: 2995.20
	clflush size	: 64
	cache_alignment	: 64
	address sizes	: 39 bits physical, 48 bits virtual
	power management:
	"""

	if exitcode != 0:
		raise Exception()

	cpuInfos = splitAtEmptyLines(stdout.split("\n"))
	retExtra = {}
	ret = []
	for group in cpuInfos:
		d = _parserColonKVP.parseLines(group)
		if "processor" not in d:
			for k, v in d.items():
				retExtra[k.lower()] = v
			continue

		if "cache_size" in d:
			d["cache_size_kb"] = ValueParser_ByteWithUnit.parse(d["cache_size"]) // 1024
			del d["cache_size"]

		if "bogomips" in d:
			d["bogomips"] = float(d["apicid"])
		elif "BogoMIPS" in d:
			d["bogomips"] = float(d["BogoMIPS"])
			del d["BogoMIPS"]

		if "bugs" in d:
			d["bugs"] = d["bugs"].split()

		if "flags" in d:
			d["flags"] = sorted(d["flags"].split())
		elif "Features" in d:
			d["flags"] = sorted(d["Features"].split())
			del d["Features"]

		# bool
		for key in [ "fpu", "fpu_exception", "wp" ]:
			if key in d:
				d[key.lower()] = d[key] == "yes"
				if key != key.lower():
					del d[key]

		# int
		for key in [ "CPU_architecture", "CPU_revision", "physical_id", "initial_apicid", "cpu_cores", "core_id", "clflush_size", "cache_alignment", "apicid" ]:
			if key in d:
				d[key.lower()] = int(d[key])
				if key != key.lower():
					del d[key]

		# float
		for key in [ "cpu_MHz" ]:
			if key in d:
				d[key.lower()] = float(d[key])
				if key != key.lower():
					del d[key]

		# str
		for key in [ "CPU_implementer", "CPU_part", "CPU_variant" ]:
			if key in d:
				d[key.lower()] = d[key]
				if key != key.lower():
					del d[key]

		d["processor"] = int(d["processor"])

		if "siblings" in d:
			d["siblings"] = int(d["siblings"])

		#jk_json.prettyPrint(d)
		ret.append(d)

	return ret, retExtra
#



#
# Returns:
#
#	[
#		{
#			"<key>": "<value>",
#			...
#		},
#		...
#	]
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_proc_cpu_info(c = None) -> typing.Tuple[list,dict]:
	stdout, stderr, exitcode = run(c, "cat /proc/cpuinfo")
	return parse_proc_cpu_info(stdout, stderr, exitcode)
#












import re

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



#
# Returns:
#
#	{
#		"install": [
#			"containerd",
#			"linux-headers-4.4.0-159",
#			"linux-headers-4.4.0-159-generic",
#			"linux-image-4.4.0-159-generic",
#			"linux-modules-4.4.0-159-generic",
#			"linux-modules-extra-4.4.0-159-generic",
#			"runc"
#		],
#		"remove": [],
#		"unnecessary": [],
#		"upgrade": [
#			"apparmor",
#			"apport",
#			"apport-gtk",
#			"apt",
#			...
#		]
#	}
#
def parse_apt_list_upgradable(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	chromium-browser/xenial-updates,xenial-security 77.0.3865.90-0ubuntu0.16.04.1 amd64 [upgradable from: 76.0.3809.100-0ubuntu0.16.04.1]
	chromium-browser-l10n/xenial-updates,xenial-updates,xenial-security,xenial-security 77.0.3865.90-0ubuntu0.16.04.1 all [upgradable from: 76.0.3809.100-0ubuntu0.16.04.1]
	chromium-codecs-ffmpeg-extra/xenial-updates,xenial-security 77.0.3865.90-0ubuntu0.16.04.1 amd64 [upgradable from: 76.0.3809.100-0ubuntu0.16.04.1]
	e2fslibs/xenial-updates,xenial-security 1.42.13-1ubuntu1.1 amd64 [upgradable from: 1.42.13-1ubuntu1]
	e2fsprogs/xenial-updates,xenial-security 1.42.13-1ubuntu1.1 amd64 [upgradable from: 1.42.13-1ubuntu1]
	initramfs-tools/xenial-updates,xenial-updates 0.122ubuntu8.15 all [upgradable from: 0.122ubuntu8.14]
	initramfs-tools-bin/xenial-updates 0.122ubuntu8.15 amd64 [upgradable from: 0.122ubuntu8.14]
	initramfs-tools-core/xenial-updates,xenial-updates 0.122ubuntu8.15 all [upgradable from: 0.122ubuntu8.14]
	libcomerr2/xenial-updates,xenial-security 1.42.13-1ubuntu1.1 amd64 [upgradable from: 1.42.13-1ubuntu1]
	libsdl2-2.0-0/xenial-updates,xenial-security 2.0.4+dfsg1-2ubuntu2.16.04.2 amd64 [upgradable from: 2.0.4+dfsg1-2ubuntu2.16.04.1]
	libsdl2-dev/xenial-updates,xenial-security 2.0.4+dfsg1-2ubuntu2.16.04.2 amd64 [upgradable from: 2.0.4+dfsg1-2ubuntu2.16.04.1]
	libss2/xenial-updates,xenial-security 1.42.13-1ubuntu1.1 amd64 [upgradable from: 1.42.13-1ubuntu1]
	linux-generic/xenial-updates,xenial-security 4.4.0.165.173 amd64 [upgradable from: 4.4.0.164.172]
	linux-headers-generic/xenial-updates,xenial-security 4.4.0.165.173 amd64 [upgradable from: 4.4.0.164.172]
	linux-image-generic/xenial-updates,xenial-security 4.4.0.165.173 amd64 [upgradable from: 4.4.0.164.172]
	linux-libc-dev/xenial-updates,xenial-security 4.4.0-165.193 amd64 [upgradable from: 4.4.0-164.192]
	tzdata/xenial-updates,xenial-updates,xenial-security,xenial-security 2019c-0ubuntu0.16.04 all [upgradable from: 2019b-0ubuntu0.16.04]
	"""
	
	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")

	upgradable = []
	for line in lines:
		m = re.match(r"^([^/]+)/([^\s]+)\s.*\[upgradable from: .*]$", line)
		if m:
			upgradable.append(m.group(1))
		else:
			if line == "Listing...":
				continue
			else:
				raise Exception("Unparsable line: " + repr(line))

	return {
		"install": [],
		"remove": [],
		"unnecessary": [],
		"upgrade": upgradable,
	}
#



#
# Returns:
#
#	{
#		"install": [],
#		"remove": [],
#		"unnecessary": [],
#		"upgrade": [
#			"containerd",
#			"linux-headers",
#			"linux-headers-generic",
#			"linux-image-generic",
#			"linux-modules-generic",
#			"linux-modules-extra-generic",
#			"runc"
#			"apparmor",
#			"apport",
#			"apport-gtk",
#			"apt",
#			...
#		]
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_apt_list_upgradable(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/apt list --upgradable")
	return parse_apt_list_upgradable(stdout, stderr, exitcode)
#























from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter(delimiter=":", valueCanBeWrappedInDoubleQuotes=False, keysReplaceSpacesWithUnderscores=False)




#
# Requires:
#	system package "sensors"
#
# Returns:
#	{
#		"acpitz-virtual-0": {
#			"adapterType": "Virtual device",
#			"device": "acpitz-virtual-0",
#			"sensorData": {
#				"temp1": {
#					"crit": 119.0,
#					"value": 27.8
#				},
#				"temp2": {
#					"crit": 119.0,
#					"value": 29.8
#				}
#			}
#		},
#		"coretemp-isa-0000": {
#			"adapterType": "ISA adapter",
#			"device": "coretemp-isa-0000",
#			"sensorData": {
#				"Core_0": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 27.0,
#					"max": 84.0
#				},
#				"Core_1": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 27.0,
#					"max": 84.0
#				},
#				"Core_2": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 25.0,
#					"max": 84.0
#				},
#				"Core_3": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 23.0,
#					"max": 84.0
#				},
#				"Physical_id_0": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 29.0,
#					"max": 84.0
#				}
#			}
#		}
#	}
#
# Notes:
# * Some WLAN adapters have a temperature sensor. If WLAN is turned off no value will be measured. Therefor a sensor data dictionary can be empty and will have no <c>value</c> key.
#
def parse_sensors(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	acpitz-virtual-0
	Adapter: Virtual device
	temp1:
	  temp1_input: 27.800
	  temp1_crit: 119.000
	temp2:
	  temp2_input: 29.800
	  temp2_crit: 119.000

	coretemp-isa-0000
	Adapter: ISA adapter
	Physical id 0:
	  temp1_input: 26.000
	  temp1_max: 84.000
	  temp1_crit: 100.000
	  temp1_crit_alarm: 0.000
	Core 0:
	  temp2_input: 24.000
	  temp2_max: 84.000
	  temp2_crit: 100.000
	  temp2_crit_alarm: 0.000
	Core 1:
	  temp3_input: 24.000
	  temp3_max: 84.000
	  temp3_crit: 100.000
	  temp3_crit_alarm: 0.000
	Core 2:
	  temp4_input: 23.000
	  temp4_max: 84.000
	  temp4_crit: 100.000
	  temp4_crit_alarm: 0.000
	Core 3:
	  temp5_input: 23.000
	  temp5_max: 84.000
	  temp5_crit: 100.000
	  temp5_crit_alarm: 0.000
	"""

	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")

	ret = {}
	for lineGroup in splitAtEmptyLines(lines):
		device = lineGroup[0]
		adapterType = lineGroup[1].split(":")[1].strip()
		sensorData = groupLinesByLeadingSpace(lineGroup[2:])

		sensorData2 = {}
		for sensorName, vGroup in sensorData.items():
			assert sensorName.endswith(":")
			sensorName = sensorName[:-1].replace(" ", "_")
			sensorData3 = {}
			for vkey, vValue in _parserColonKVP.parseLines(vGroup).items():
				if vkey.startswith("temp"):
					sensorData3["sensor"] = "temp"
					pos = vkey.find("_")
					vkey = vkey[pos+1:]
				elif vkey.startswith("fan"):
					sensorData3["sensor"] = "fan"
					pos = vkey.find("_")
					vkey = vkey[pos+1:]
				if vkey == "input":
					vkey = "value"
				sensorData3[vkey] = float(vValue)
			sensorData2[sensorName] = sensorData3

		ret[device] = {
			"device": device,
			"adapterType": adapterType,
			"sensorData": sensorData2,
		}

	return ret
#



#
# Requires:
#	system package "sensors"
#
# Returns:
#	{
#		"acpitz-virtual-0": {
#			"adapterType": "Virtual device",
#			"device": "acpitz-virtual-0",
#			"sensorData": {
#				"temp1": {
#					"crit": 119.0,
#					"value": 27.8
#				},
#				"temp2": {
#					"crit": 119.0,
#					"value": 29.8
#				}
#			}
#		},
#		"coretemp-isa-0000": {
#			"adapterType": "ISA adapter",
#			"device": "coretemp-isa-0000",
#			"sensorData": {
#				"Core_0": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 27.0,
#					"max": 84.0
#				},
#				"Core_1": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 27.0,
#					"max": 84.0
#				},
#				"Core_2": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 25.0,
#					"max": 84.0
#				},
#				"Core_3": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 23.0,
#					"max": 84.0
#				},
#				"Physical_id_0": {
#					"crit": 100.0,
#					"crit_alarm": 0.0,
#					"value": 29.0,
#					"max": 84.0
#				}
#			}
#		}
#	}
#
def get_sensors(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/sensors -u")
	return parse_sensors(stdout, stderr, exitcode)
#










#
# TODO: Migrate this to jk_utils and jk_valueformatting.
#




import jk_console




#
# Formats a value provided in bits per second.
#
# @param	float value			A value (measured in bits per second).
# @returns	float value			The scaled value.
# @returns	str unit			The unit of the value, e.g. "Gbit/s".
#
def formatBitsPerSecond(value:float, floatPrecision:bool = False) -> tuple:
	m = 1000
	if floatPrecision:
		if value >= m:
			if value >= m*m:
				if value >= m*m*m:
					value = int(value / (m*m*m) * 10 + 0.5) / 10
					unit = "Gbit/s"
				else:
					value = int(value / (m*m) * 10 + 0.5) / 10
					unit = "Mbit/s"
			else:
				value = value / m
				unit = "Kbit/s"
		else:
			unit = "bit/s"
	else:
		if value >= m:
			if value >= m*m:
				if value >= m*m*m:
					value = value // (m*m*m)
					unit = "Gbit/s"
				else:
					value = value // (m*m)
					unit = "Mbit/s"
			else:
				value = value // m
				unit = "Kbit/s"
		else:
			unit = "bit/s"
	return value, unit
#



#
# Same as <c>formatBitsPerSecond()</c> but does not return a tuple but a string.
#
def formatBitsPerSecondS(value:float, floatPrecision:bool = False) -> str:
	value, unit = formatBitsPerSecond(value, floatPrecision)
	return str(value) + " " + unit
#



#
# Formats a value provided in bytes.
#
# @param	float value			A value (measured in bytes).
# @returns	float value			The scaled value.
# @returns	str unit			The unit of the value, e.g. "Gbyte".
#
def formatBytes(value:float, floatPrecision:bool = True) -> tuple:
	m = 1024
	if floatPrecision:
		if value >= m:
			if value >= m*m:
				if value >= m*m*m:
					if value >= m*m*m*m:
						value = int(value / (m*m*m*m) * 10 + 0.5) / 10
						unit = "TByte"
					else:
						value = int(value / (m*m*m) * 10 + 0.5) / 10
						unit = "GByte"
				else:
					value = int(value / (m*m) * 10 + 0.5) / 10
					unit = "MByte"
			else:
				value = int(value / m * 10 + 0.5) / 10
				unit = "KByte"
		else:
			unit = "Byte"
	else:
		if value >= m:
			if value >= m*m:
				if value >= m*m*m:
					if value >= m*m*m*m:
						value = value // m*m*m*m
						unit = "TByte"
					else:
						value = value // m*m*m
						unit = "GByte"
				else:
					value = value // m*m
					unit = "MByte"
			else:
				value = value // m
				unit = "KByte"
		else:
			value = int(value)
			unit = "Byte"
	return value, unit
#

def formatBytesS(value:float, floatPrecision:bool = True) -> str:
	value, unit = formatBytes(value, floatPrecision)
	return str(value) + " " + unit
#



#
# Formats a value provided in Herz.
#
# @param	float value			A value (measured in Herz).
# @returns	float value			The scaled value.
# @returns	str unit			The unit of the value, e.g. "GHz".
#
def formatFrequency(value:float, floatPrecision:bool = True) -> tuple:
	if floatPrecision:
		if value >= 1000:
			if value >= 1000000:
				if value >= 1000000000:
					value = (value / 1000000000 * 10 + 0.5) / 10
					unit = "GHz"
				else:
					value = (value / 1000000 * 10 + 0.5) / 10
					unit = "MHz"
			else:
				value = (value / 1000 * 10 + 0.5) / 10
				unit = "KHz"
		else:
			unit = "Hz"
	else:
		if value >= 1000:
			if value >= 1000000:
				if value >= 1000000000:
					value = value // 1000000000
					unit = "GHz"
				else:
					value = value // 1000000
					unit = "MHz"
			else:
				value = value // 1000
				unit = "KHz"
		else:
			unit = "Hz"
	return value, unit
#

def formatFrequencyS(value:float, floatPrecision:bool = True) -> str:
	value, unit = formatFrequency(value, floatPrecision)
	return str(value) + " " + unit
#



#
# Formats a value provided in Herz.
#
# @param	float value1			A value (measured in Herz).
# @param	float value2			A value (measured in Herz).
# @returns	float value1			The scaled value 1.
# @returns	float value2			The scaled value.2.
# @returns	str unit				The unit of the value, e.g. "GHz".
#
def formatFrequencyRange(value1:float, value2:float, floatPrecision:bool = True) -> tuple:
	if floatPrecision:
		value = max(value1, value2)
		if value >= 1000:
			if value >= 1000000:
				if value >= 1000000000:
					value1 = (value1 / 1000000000 * 10 + 0.5) / 10
					value2 = (value2 / 1000000000 * 10 + 0.5) / 10
					unit = "GHz"
				else:
					value1 = (value1 / 1000000 * 10 + 0.5) / 10
					value2 = (value2 / 1000000 * 10 + 0.5) / 10
					unit = "MHz"
			else:
				value1 = (value1 / 1000 * 10 + 0.5) / 10
				value2 = (value2 / 1000 * 10 + 0.5) / 10
				unit = "KHz"
		else:
			unit = "Hz"
	else:
		value = min(value1, value2)
		if value >= 1000:
			if value >= 1000000:
				if value >= 1000000000:
					value1 = value1 // 1000000000
					value2 = value2 // 1000000000
					unit = "GHz"
				else:
					value1 = value1 // 1000000
					value2 = value2 // 1000000
					unit = "MHz"
			else:
				value1 = value1 // 1000
				value2 = value2 // 1000
				unit = "KHz"
		else:
			value1 = int(value1)
			value2 = int(value2)
			unit = "Hz"
	return value1, value2, unit
#

def formatFrequencyRangeS(value1:float, value2:float, floatPrecision:bool = True) -> str:
	value1, value2, unit = formatFrequencyRange(value1, value2, floatPrecision)
	return str(value1) + "-" + str(value2) + " " + unit
#



def formatPercent(value:float, total:float) -> str:
	return str(int(value / total * 1000) / 10) + "%"
#



#
# @param	float fromColorR		Red spectrum part ranging from 0 to 1
# @param	float fromColorG		Green spectrum part ranging from 0 to 1
# @param	float fromColorB		Blue spectrum part ranging from 0 to 1
# @param	float toColorR			Red spectrum part ranging from 0 to 1
# @param	float toColorG			Green spectrum part ranging from 0 to 1
# @param	float toColorB			Blue spectrum part ranging from 0 to 1
# @param	int length				The number of color values to create
# @return	list					An array of strings with color escape codes
#
def _createColorSpectrum(fromColorR:float, fromColorG:float, fromColorB:float, toColorR:float, toColorG:float, toColorB:float, length:int):
	ret = []
	for i in range(0, length):
		v = i / (length -1)
		vv = 1 - v
		r = fromColorR * vv + toColorR * v
		g = fromColorG * vv + toColorG * v
		b = fromColorB * vv + toColorB * v
		ret.append(jk_console.Console.ForeGround.rgb1(r, g, b))
	return ret
#

#
# Build a graph showing fill grade in percentage
#
# @param	float value			The current value. This can be an arbitrary value greater than zero and smaller than <c>total</c>.
# @param	float total			The maximum value. This can be an arbitrary value greater than <c>value</c>.
# @param	int length			The length of the bar to produce.
#
def formatPercentGraphC(value:float, total:float, length:int = 40) -> str:
	n = int(value / total * length)
	s = "#" * n

	colorSpectrum1 = _createColorSpectrum(0.7, 0.7, 0.7, 0.7, 0.7, 0, int(length * 0.1))
	colorSpectrum2 = _createColorSpectrum(0.7, 0.7, 0,   1,   0.5, 0, int(length * 0.1))
	colorSpectrum3 = _createColorSpectrum(1,   0.5, 0,   1,   0,   0, int(length * 0.1))
	colorSpectrum4 = _createColorSpectrum(1,   0,   0,   1,   0,   0, int(length * 0.05))
	colorSpectrum = colorSpectrum1 + colorSpectrum2 + colorSpectrum3 + colorSpectrum4
	posStart = length - len(colorSpectrum)

	bColorAdded = False
	for pos in range(len(s) - 1, -1, -1):
		i = pos - posStart
		if i < 0:
			break
		s = s[:pos] + colorSpectrum[i] + s[pos:]
		bColorAdded = True

	if bColorAdded:
		s += jk_console.Console.RESET

	s += ":" * (length - n)
	return s
#

def formatTemperatureGraphC(value:float, maximum:float = 100, length:int = 40) -> str:
	assert isinstance(value, (int, float))
	assert isinstance(maximum, (int, float))
	assert isinstance(length, int)

	orgValue = value
	if value < 0:
		value = 0
	if value > maximum:
		value = maximum

	n = int(value / maximum * length)
	s = "#" * n

	colorSpectrum1 = _createColorSpectrum(0.7, 0.7, 0.7, 0.7, 0.7, 0, int(length * 0.23))
	colorSpectrum2 = _createColorSpectrum(0.7, 0.7, 0,   1,   0.5, 0, int(length * 0.23))
	colorSpectrum3 = _createColorSpectrum(1,   0.5, 0,   1,   0,   0, int(length * 0.23))
	colorSpectrum = colorSpectrum1 + colorSpectrum2 + colorSpectrum3
	posStart = length - len(colorSpectrum)

	firstColor = ""
	bColorAdded = False
	for pos in range(len(s) - 1, -1, -1):
		i = pos - posStart
		if i < 0:
			break
		s = s[:pos] + colorSpectrum[i] + s[pos:]
		if not bColorAdded:
			firstColor = colorSpectrum[i]
			bColorAdded = True

	if bColorAdded:
		s += jk_console.Console.RESET

	s += ":" * (length - n)
	return s + " " + firstColor + str(int(orgValue + 0.5)) + "°C" + jk_console.Console.RESET
#

def convertSecondsToHumanReadableDuration(seconds:float):
	milliseconds = (seconds - int(seconds)) * 1000
	seconds = int(seconds)

	minutes = seconds // 60
	seconds = seconds % 60

	hours = minutes // 60
	minutes = minutes % 60

	days = hours // 24
	hours = hours % 24

	return days, hours, minutes, seconds, milliseconds
#
















import os
import subprocess

try:
	from fabric import Connection
except ImportError as ee:
	pass



_debuggingEnabled = False

def enableDebugging():
	global _debuggingEnabled

	_debuggingEnabled = True
#





#
# Run a command locally or remotely.
# If a "cat <file>" is to be invoked *and* this is to be invoked locally, this method will detect this. In that case instead of running "cat" it will fall back to a regular file read
# for efficiency. Therefore you can access data on local and remote systems in a uniform way without spending too much thoughts on efficiency.
#
def run(c, command, failOnNonZeroExitCode:bool = True):
	global _debuggingEnabled

	if c is None:
		if command.startswith("cat "):
			filePath = command[4:]
			if _debuggingEnabled:
				print("Using standard file reading for command: " + repr(command))
			if os.path.isfile(filePath):
				with open(filePath, "r") as f:
					return f.read(), "", 0
			else:
				raise Exception("No such file: " + repr(filePath))
		if _debuggingEnabled:
			print("Invoking via subprocess: " + repr(command))
		p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
		binStdOut, binStdErr = p.communicate()
		stdOut = binStdOut.decode("utf-8")
		stdErr = binStdErr.decode("utf-8")
		if _debuggingEnabled:
			print("exit status:", p.returncode)
			print("stdout:")
			for line in stdOut.split("\n"):
				print("\t" + repr(line))
			print("stderr:")
			for line in stdErr.split("\n"):
				print("\t" + repr(line))
		if failOnNonZeroExitCode and p.returncode > 0:
			raise Exception("Command failed with exit code " + str(p.returncode) + ": " + repr(command))
		return stdOut, stdErr, p.returncode

	if (c.__class__.__name__ == "Connection") and (c.__class__.__module__ in [ "fabric", "fabric.connection" ]):
		if _debuggingEnabled:
			print("Invoking via fabric: " + repr(command))
		r = c.run(command, hide=True)
		if _debuggingEnabled:
			print("exit status:", r.exited)
			print("stdout:")
			for line in r.stdout.split("\n"):
				print("\t" + repr(line))
			print("stderr:")
			for line in r.stderr.split("\n"):
				print("\t" + repr(line))
		if failOnNonZeroExitCode and r.exited > 0:
			raise Exception("Command failed with exit code " + str(r.exited) + ": " + repr(command))
		return r.stdout, r.stderr, r.exited

	raise Exception("Sorry, I don't know about " + repr(c.__class__) + " objects for parameter c.")
#















#!/usr/bin/python3


import os
import sys
import re

import jk_console
import jk_sysinfo
import jk_json
import jk_flexdata
import jk_logging
from jk_typing import *
import jk_argparsing







class SysInfoOption(object):

	def __init__(self, longOption:str, description:str, function, **kwargs):
		self.longOption = longOption
		self.description = description
		self.function = function
		self.extraArgs = kwargs
	#

	def onOptionEnable(self, argOption, argOptionArguments, parsedArgs):
		parsedArgs.optionData.set(self.longOption, True)
	#

	def onOptionDisable(self, argOption, argOptionArguments, parsedArgs):
		parsedArgs.optionData.set(self.longOption, False)
	#

	def register(self, ap:jk_argparsing.ArgsParser):
		ap.optionDataDefaults.set(self.longOption, False)
		c = ap.createOption(None, self.longOption, self.description + " (enable)").onOption = self.onOptionEnable
		c = ap.createOption(None, self.longOption + "-0", self.description + " (disable)").onOption = self.onOptionDisable
	#

	def run(self, c):
		return self.function(c, **self.extraArgs)
	#

#





ALL_SYSINFO_OPTIONS = [
	SysInfoOption("i-accesspoints", "Get data about access points", jk_sysinfo.get_accesspoints),
	SysInfoOption("i-apt-list-upgradable", "Get information about upgradable packages", jk_sysinfo.get_apt_list_upgradable),
	SysInfoOption("i-bios-info", "Get information about the BIOS", jk_sysinfo.get_bios_info),
	SysInfoOption("i-cpu-info", "Get information about the CPU", jk_sysinfo.get_cpu_info),
	SysInfoOption("i-date", "Get date (local)", jk_sysinfo.get_date, utc=False),
	SysInfoOption("i-date-utc", "Get date (UTC)", jk_sysinfo.get_date, utc=True),
	SysInfoOption("i-df", "Get disk space information", jk_sysinfo.get_df),
	SysInfoOption("i-dpkg-list", "Get information about installed packages", jk_sysinfo.get_dpkg_list),
	SysInfoOption("i-etc-hostname", "Get hostname information", jk_sysinfo.get_etc_hostname),
	SysInfoOption("i-etc-os-release", "Get information about the OS as stored in /etc/os/release", jk_sysinfo.get_etc_os_release),
	SysInfoOption("i-ifconfig", "Get information about the network interfaces as provided by 'ifconfig'", jk_sysinfo.get_ifconfig),
	SysInfoOption("i-lsb-release-a", "Get information about the current linux distribution", jk_sysinfo.get_lsb_release_a),
	SysInfoOption("i-lsblk", "Get information about block devices", jk_sysinfo.get_lsblk),
	SysInfoOption("i-lshw", "Get information about the hardware as provided by 'lshw'", jk_sysinfo.get_lshw),
	SysInfoOption("i-motherboard-info", "Get information about the motherboard", jk_sysinfo.get_motherboard_info),
	SysInfoOption("i-mount", "Get information about the mounted devices", jk_sysinfo.get_mount),
	SysInfoOption("i-needs-reboot", "Check if a reboot is required", jk_sysinfo.get_needs_reboot),
	SysInfoOption("i-net-info", "Retrieve various information about network devices", jk_sysinfo.get_net_info),
	SysInfoOption("i-pip3-list", "Retrieve a list of installed python packages", jk_sysinfo.get_pip3_list),
	SysInfoOption("i-proc-cpu-info", "Retrieve information about the CPU as provided by /proc/cpuinfo", jk_sysinfo.get_proc_cpu_info),
	SysInfoOption("i-proc-load-avg", "Retrieve information about the system load as provided by /proc/loadavg", jk_sysinfo.get_proc_load_avg),
	SysInfoOption("i-proc-meminfo", "Retrieve information about the memory usage as provided by /proc/meminfo", jk_sysinfo.get_proc_meminfo),
	SysInfoOption("i-ps", "Get information about processes", jk_sysinfo.get_ps),
	SysInfoOption("i-sensors", "Get information about processes", jk_sysinfo.get_sensors),
	SysInfoOption("i-systemctl-units", "Get information about all systemctrl units", jk_sysinfo.get_systemctl_units),
	SysInfoOption("i-uptime", "Get information about uptime", jk_sysinfo.get_uptime),
	SysInfoOption("i-user-info", "Get information about users", jk_sysinfo.get_user_info),
	SysInfoOption("i-vcgencmd", "Get information about a Raspberry Pi", jk_sysinfo.get_vcgencmd),
]
if os.getuid() == 0:
	ALL_SYSINFO_OPTIONS.extend([
		SysInfoOption("i-etc-group", "Get all data from /etc/group and /etc/gshadow", jk_sysinfo.get_etc_group),
		SysInfoOption("i-etc-passwd", "Get all data from /etc/passwd and /etc/shadow", jk_sysinfo.get_etc_passwd),
	])

ALL_SYSINFO_OPTIONS_MAP = {
	opt.longOption:opt for opt in ALL_SYSINFO_OPTIONS
}

def onOptionAll(argOption, argOptionArguments, parsedArgs):
	for opt in ALL_SYSINFO_OPTIONS:
		parsedArgs.optionData.set(opt.longOption, True)
#

def onOptionAllStd(argOption, argOptionArguments, parsedArgs):
	for opt in ALL_SYSINFO_OPTIONS:
		if opt.longOption in [ "i-vcgencmd" ]:
			continue
		parsedArgs.optionData.set(opt.longOption, True)
#

def onOptionAllVM(argOption, argOptionArguments, parsedArgs):
	for opt in ALL_SYSINFO_OPTIONS:
		if opt.longOption in [ "i-vcgencmd" ]:
			continue
		if opt.longOption in [ "i-sensors" ]:
			continue
		parsedArgs.optionData.set(opt.longOption, True)
#

def onOptionAllRPi(argOption, argOptionArguments, parsedArgs):
	for opt in ALL_SYSINFO_OPTIONS:
		if opt.longOption in []:
			continue
		parsedArgs.optionData.set(opt.longOption, True)
#





ap = jk_argparsing.ArgsParser(os.path.basename(__file__), "Display system information.")

ap.optionDataDefaults.set("help", False)
ap.optionDataDefaults.set("color", True)

ap.createOption('h', 'help', "Display this help text.").onOption = \
	lambda argOption, argOptionArguments, parsedArgs: \
		parsedArgs.optionData.set("help", True)
ap.createOption(None, 'no-color', "Dont' use colors in output.").onOption = \
	lambda argOption, argOptionArguments, parsedArgs: \
		parsedArgs.optionData.set("colors", False)
for opt in ALL_SYSINFO_OPTIONS:
	opt.register(ap)
ap.createOption(None, 'i-all', "Use all system information modules.").onOption = onOptionAll
ap.createOption(None, 'i-all-std', "Use all system information modules (Regular *nix Machines).").onOption = onOptionAllStd
ap.createOption(None, 'i-all-rpi', "Use all system information modules (Raspberry Pi).").onOption = onOptionAllRPi
ap.createOption(None, 'i-all-vm', "Use all system information modules (Virtual Machines).").onOption = onOptionAllVM

ap.createAuthor("Jürgen Knauth", "jk@binary-overflow.de")
ap.setLicense("Apache", YEAR = 2020, COPYRIGHTHOLDER = "Jürgen Knauth")

ap.createReturnCode(0, "Everything is okay.")
ap.createReturnCode(1, "An error occurred.")






parsedArgs = ap.parse()
#parsedArgs.dump()

if parsedArgs.optionData["help"]:
	ap.showHelp()
	sys.exit(0)

"""
cmdName, cmdArgs = parsedArgs.parseNextCommand()
if cmdName is None:
	ap.showHelp()
	sys.exit(0)
"""

if parsedArgs.optionData["color"]:
	log = jk_logging.ConsoleLogger.create(logMsgFormatter = jk_logging.COLOR_LOG_MESSAGE_FORMATTER, printToStdErr = True)
else:
	log = jk_logging.ConsoleLogger.create(printToStdErr = True)

bSuccess = True

ret = {}
for opt in ALL_SYSINFO_OPTIONS:
	if parsedArgs.optionData[opt.longOption]:
		try:
			ret[opt.longOption] = opt.run(None)
		except Exception as ee:
			# there has been an error
			ret[opt.longOption] = None
			# log.error("Failed to retrieve data for: " + opt.longOption)
			log.exception(ee)

jk_json.prettyPrint(ret)

sys.exit(0 if bSuccess else 1)















import re

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run

from jk_version import Version




#
# Returns:
#
#	{
#		...
#		"netcat-openbsd": "1.105-7ubuntu1",
#		"netpbm": "2:10.0-15.3",
#		"nettle-dev", "3.2-1ubuntu0.16.04.1",
#		...
#	}
#
def parse_dpkg_list(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	...
	netcat-openbsd\t1.105-7ubuntu1
	netpbm\t2:10.0-15.3
	nettle-dev\t3.2-1ubuntu0.16.04.1
	...
	"""
	
	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")

	ret = {}
	for line in lines:
		name, version = line.split("\t")
		p = name.find(":")
		if p > 0:
			name = name[:p]
		ret[name] = version

	return ret
#



#
# Returns:
#
#	{
#		...
#		"netcat-openbsd": "1.105-7ubuntu1",
#		"netpbm": "2:10.0-15.3",
#		"nettle-dev", "3.2-1ubuntu0.16.04.1",
#		...
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_dpkg_list(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/dpkg-query -W")
	return parse_dpkg_list(stdout, stderr, exitcode)
#


















